#!/bin/bash

GCMC="../src/gcmc"
OPTS="-I ../library"

TESTOUT="testoutput.ngc"

TESTFILES=( \
	"math.test.gcmc" \
	"units.test.gcmc" \
	"include.test.gcmc" \
	"vector.test.gcmc" \
	"star.test.gcmc" \
	"arccw.test.gcmc" \
	"vcreate.test.gcmc" \
	"floatint.test.gcmc" \
	"headtail.test.gcmc" \
	"break.test.gcmc" \
	"continue.test.gcmc" \
	)

TESTRETURNS=( \
	0 \
	0 \
	0 \
	0 \
	0 \
	0 \
	0 \
	0 \
	0 \
	0 \
	0 \
	)

ERRORS=0

# Sanity check
if [ ${#TESTFILES[@]} -ne ${#TESTRETURNS[@]} ]; then
	echo "Error: Number of testfiles not equal to number of return values. Tests aborted."
	exit 1
fi

# Test all files for results
for((i=0; i<${#TESTFILES[@]}; i++))
do
	TESTFILE="${TESTFILES[$i]}"
	TESTEXPECT="$(basename ${TESTFILE} ".gcmc").result"
	TESTRETURN=${TESTRETURNS[$i]}
	if [ ! -r "${TESTFILE}" ]; then
		echo "Error: '${TESTFILE}' is missing"
		ERRORS=$(($ERRORS + 1))
		continue;
	fi
	if [ ! -r "${TESTEXPECT}" ]; then
		echo "Error: '${TESTFILE}' has missing result file '${TESTEXPECT}'"
		ERRORS=$(($ERRORS + 1))
		continue;
	fi
	$GCMC ${OPTS} -q "${TESTFILE}" > "${TESTOUT}" 2>&1
	RV=$?
	if [ $RV -ne $TESTRETURN ]; then
		echo "Error: '${TESTFILE}' compiled with unexpected return value $RV"
		ERRORS=$(($ERRORS + 1))
	fi
	if ! diff -q "${TESTEXPECT}" "${TESTOUT}"; then
		echo "Error: '${TESTFILE}' has different result than expected. Differences:"
		diff -u "${TESTEXPECT}" "${TESTOUT}"
	fi
done

rm -f "${TESTOUT}"
exit $ERRORS
